@@ -1,6 +1,6 @@
 /*-------------------------------------------------------------------------
 |   rxtx is a native interface to i2c ports in java.
-|   Copyright 1997-2002 by Trent Jarvi taj@www.linux.org.uk
+|   Copyright 1997-2004 by Trent Jarvi taj@www.linux.org.uk
 |
 |   This library is free software; you can redistribute it and/or
 |   modify it under the terms of the GNU Library General Public
@@ -16,12 +16,12 @@
 |   License along with this library; if not, write to the Free
 |   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 --------------------------------------------------------------------------*/
-#if defined(__MWERKS__)//dima
-#include "I2C.h"//dima
-#else//dima
+#if defined(__MWERKS__) /* dima */
+#include "I2C.h" /* dima */
+#else /* dima */
 #include "config.h"
 #include "gnu_io_I2C.h"
-#endif//dima
+#endif /* dima */
 #include <time.h>
 #include <unistd.h>
 #include <stdio.h>
@@ -196,9 +196,9 @@ JNIEXPORT void JNICALL Java_gnu_io_I2CPort_nativeSetI2CPortParams(
 	int cspeed = translate_speed( env, speed );
 	if( !cspeed ) return;
 	if( tcgetattr( fd, &ttyset ) < 0 ) goto fail;
-	if( !translate_data_bits( env, (int *)&(ttyset.c_cflag), dataBits ) ) return;//dima darwin defime c_cflag as unsigned long
-	if( !translate_stop_bits( env, (int *)&(ttyset.c_cflag), stopBits ) ) return;//dima darwin defime c_cflag as unsigned long
-	if( !translate_parity( env, (int *)&(ttyset.c_cflag), parity ) ) return;//dima darwin defime c_cflag as unsigned long
+	if( !translate_data_bits( env, (int *)&(ttyset.c_cflag), dataBits ) ) return; /* dima darwin defime c_cflag as unsigned long */
+	if( !translate_stop_bits( env, (int *)&(ttyset.c_cflag), stopBits ) ) return; /* dima darwin defime c_cflag as unsigned long */
+	if( !translate_parity( env, (int *)&(ttyset.c_cflag), parity ) ) return;/* dima darwin defime c_cflag as unsigned long */
 #ifdef __FreeBSD__
 	if( cfsetspeed( &ttyset, cspeed ) < 0 ) goto fail;
 #else
@@ -409,7 +409,7 @@ JNIEXPORT void JNICALL Java_gnu_io_I2CPort_writeArray( JNIEnv *env,
 	for( i = 0; i < count; i++ ) bytes[ i ] = body[ i + offset ];
 	(*env)->ReleaseByteArrayElements( env, jbarray, body, 0 );
 	do {
-		result=write (fd, bytes + total + offset, count - total);//dima
+		result=write (fd, bytes + total + offset, count - total); /* dima */
 		if(result >0){
 			total += result;
 		}
@@ -837,7 +837,7 @@ JNIEXPORT jint JNICALL Java_gnu_io_I2CPort_readArray( JNIEnv *env,
 	int fd = get_java_var( env, jobj, "fd", "I" );
 	int timeout = get_java_var( env, jobj, "timeout", "I" );
 
-	if( length > SSIZE_MAX || length < 0 ) {
+	if( (size_t) length > SSIZE_MAX || length < 0 ) {
 		throw_java_exception( env, ARRAY_INDEX_OUT_OF_BOUNDS,
 			"readArray", "Invalid length" );
 		return -1;
@@ -958,7 +958,7 @@ JNIEXPORT void JNICALL Java_gnu_io_I2CPort_eventLoop( JNIEnv *env,
 	jboolean interrupted = 0;
 	jclass jclazz, jthread;
 	jclazz = (*env)->GetObjectClass( env, jobj );
-	fd = fd = get_java_var(env, jobj, "fd", "I");
+	fd = get_java_var(env, jobj, "fd", "I");
 	method = (*env)->GetMethodID( env, jclazz, "sendEvent", "(IZ)V" );
 	jthread = (*env)->FindClass( env, "java/lang/Thread" );
 	interrupt = (*env)->GetStaticMethodID( env, jthread, "interrupted", "()Z" );
