@@ -1,6 +1,6 @@
 /*-------------------------------------------------------------------------
 |   rxtx is a native interface to serial ports in java.
-|   Copyright 1997-2002 by Trent Jarvi taj@www.linux.org.uk.
+|   Copyright 1997-2004 by Trent Jarvi taj@www.linux.org.uk.
 |
 |   This library is free software; you can redistribute it and/or
 |   modify it under the terms of the GNU Library General Public
@@ -36,7 +36,8 @@ public class RXTXVersion
 	private static String Version;
 
 	static {
-		Version = "RXTX-2.1-6";
+		System.loadLibrary( "rxtxSerial" );
+		Version = "RXTX-2.1-7";
 	}
 	/**
 	*  static method to return the current version of RXTX
@@ -47,4 +48,5 @@ public class RXTXVersion
 	{
 		return(Version);
 	}
+	public static native String nativeGetVersion();
 }
