@@ -1,6 +1,6 @@
 /*-------------------------------------------------------------------------
 |   rxtx is a native interface to serial ports in java.
-|   Copyright 1997-2002 by Trent Jarvi taj@www.linux.org.uk.
+|   Copyright 1997-2004 by Trent Jarvi taj@www.linux.org.uk.
 |
 |   This library is free software; you can redistribute it and/or
 |   modify it under the terms of the GNU Library General Public
@@ -103,13 +103,17 @@ public abstract class SerialPort extends CommPort {
 	public abstract String getUARTType()
 		throws UnsupportedCommOperationException;
 	public abstract boolean setBaudBase(int BaudBase)
-		throws UnsupportedCommOperationException;
+		throws UnsupportedCommOperationException,
+		IOException;
 	public abstract int getBaudBase()
-		throws UnsupportedCommOperationException;
+		throws UnsupportedCommOperationException,
+		IOException;
 	public abstract boolean setDivisor(int Divisor)
-		throws UnsupportedCommOperationException;
+		throws UnsupportedCommOperationException,
+		IOException;
 	public abstract int getDivisor()
-		throws UnsupportedCommOperationException;
+		throws UnsupportedCommOperationException,
+		IOException;
 	public abstract boolean setLowLatency()
 		throws UnsupportedCommOperationException;
 	public abstract boolean getLowLatency()
